OBJECTS = {
    "X_train": "dataframe",
    "X_test": "dataframe",
    "y_train": "dataframe",
    "y_test": "dataframe",
    "model": "model",
    "trained_model": "model",
    "prediction": "dataframe",
    "y_pred": "dataframe",
}

RECOGNIZED_DTYPES = ["dataframe", "primitive", "other"]
